@@ -42,7 +42,9 @@ type uid_type =
 | TigerTree of TigerTree.t
 | Md5Ext of Md5Ext.t     (* for Fasttrack *)
 | BTUrl of Sha1.t
-
+| FileTP of Md4.t
+| NoUid
+  
 and file_uid_id = 
 | BITPRINT
 | SHA1
@@ -50,31 +52,52 @@ and file_uid_id =
 | MD5
 | MD5EXT
 | TIGER
+| FILETP
 
-let string_of_uid uid = 
+let string_of_uid_sep uid sep =
   match uid with
-    Bitprint (sha1,ttr) -> 
-      Printf.sprintf "urn:bitprint:%s.%s" (Sha1.to_string sha1)
-      (TigerTree.to_string ttr)
-  | Sha1 sha1 -> 
-      Printf.sprintf "urn:sha1:%s"  (Sha1.to_string sha1)
-  | Ed2k ed2k -> 
-      Printf.sprintf "urn:ed2k:%s" (Md4.to_string ed2k)
-  | Md5 md5 -> 
-      Printf.sprintf "urn:md5:%s" (Md5.to_string md5)
+    Bitprint (sha1,ttr) ->
+      "urn" ^ sep ^ "bitprint" ^ sep ^
+        (Sha1.to_string sha1) ^ "." ^
+          (TigerTree.to_string ttr)
+  | Sha1 sha1 ->
+      "urn" ^ sep ^ "sha1" ^ sep ^ (Sha1.to_string sha1)
+  | Ed2k ed2k ->
+      "urn" ^ sep ^ "ed2k" ^ sep ^ (Md4.to_string ed2k)
+  | Md5 md5 ->
+      "urn" ^ sep ^ "md5" ^ sep ^ (Md5.to_string md5)
   | TigerTree ttr -> 
-      Printf.sprintf "urn:ttr:%s"  (TigerTree.to_string ttr)
-  | Md5Ext md5 -> 
-      Printf.sprintf "urn:sig2dat:%s" (Md5Ext.to_base32 md5)
+      "urn" ^ sep ^ "ttr" ^ sep ^ (TigerTree.to_string ttr)
+  | Md5Ext md5 ->
+      "urn" ^ sep ^ "sig2dat" ^ sep ^
+        (Md5Ext.to_base32 md5)
   | BTUrl url ->
-      Printf.sprintf "urn:bt:%s" (Sha1.to_string url)
+      "urn" ^ sep ^ "bt" ^ sep ^ (Sha1.to_string url)
+  | FileTP file ->
+      "urn" ^ sep ^ "filetp" ^ sep ^ (Md4.to_string file)
+  | NoUid -> ""
 
-(* Maybe it would be better to raise an exception for errors ? *)      
+let string_of_uid uid =
+  string_of_uid_sep uid ":"
+
+(* This should be used for temp. files, because for some
+   systems the character ":" may be illegal for filenames.
+ *)
+let file_string_of_uid uid =
+  string_of_uid_sep uid "_"
+
+exception Illegal_urn of string
 let uid_of_string s =
   let s = String.lowercase s in
-  let (urn, rem) = String2.cut_at s ':' in
-  if urn <> "urn" then failwith "Illegal UID";
-  let (sign, rem) = String2.cut_at rem ':' in
+  let urn = String2.before s 4 in
+  let rem = String2.after s 4 in
+  let sep = ref ':' in
+  (match urn with
+   | "urn:" -> sep := ':'
+   | "urn_" -> sep := '_'
+   | _ -> raise (Illegal_urn (s ^ " at " ^ urn ^ " should be urn: or urn_"))
+  );
+  let (sign, rem) = String2.cut_at rem !sep in
   match sign with
   | "ed2k" -> Ed2k (Md4.of_string rem)
   | "bitprint" | "bp" -> 
@@ -86,19 +109,21 @@ let uid_of_string s =
   | "tree" ->
       let (tiger, rem) = String2.cut_at rem ':' in
       if tiger <> "tiger" then 
-        failwith (Printf.sprintf "Illformed URN [%s]" s);
+        raise (Illegal_urn (s ^ " at " ^ tiger ^ " should be tiger:"));
       TigerTree (TigerTree.of_string rem)
   | "ttr" -> TigerTree (TigerTree.of_string rem)
   | "md5" ->  Md5 (Md5.of_string rem)
   | "sig2dat" -> Md5Ext (Md5Ext.of_base32 rem)
   | "bt" | "bittorrent" -> 
       BTUrl (Sha1.of_string rem)
-  | _ -> failwith "Illegal UID"
-      
+  | "filetp" -> FileTP (Md4.of_string rem)
+  | _ -> raise (Illegal_urn (s ^ " at " ^ sign ^ " is not known"))
+
 module Uid : sig
-    type t     
+    type t
     val create : uid_type -> t
     val to_string : t -> string
+    val to_file_string : t -> string
     val of_string : string -> t
     val to_uid : t -> uid_type
     val mem : t -> t list -> bool
@@ -113,49 +138,38 @@ module Uid : sig
   end = struct
     
     type t = {
-        mutable uid_string : string option;
-        mutable uid_type : uid_type option;
+        mutable uid_string : string;
+        mutable uid_type : uid_type;
       }
     
     let no = {
-        uid_type = None;
-        uid_string = None;
+        uid_type = NoUid;
+        uid_string = "";
       }
     
     let create uid = {
-        uid_string = None ;
-        uid_type = Some uid;
+        uid_string = string_of_uid uid ;
+        uid_type = uid;
       }
     
     let of_string s = 
-      if s = "" then no else
-        {
-          uid_string = Some s;
-          uid_type = None;
+      let uid = uid_of_string s in
+      let s = string_of_uid uid in
+      {
+        uid_string = s;
+        uid_type = uid;
         }
         
-    let to_string t =
-      match t.uid_string, t.uid_type with 
-        Some s, _ -> s
-      | None, Some uid -> 
-          let s = string_of_uid uid in
-          t.uid_string <- Some s;
-          s
-      | None, None -> ""
+    let to_string t = t.uid_string
     
-    let to_uid t = 
-      match t.uid_type, t.uid_string with 
-        Some uid, _ -> uid
-      | None, Some s -> 
-          let uid = uid_of_string s in
-          t.uid_type <- Some uid;
-          uid
-      | None, None -> assert false
+    let to_file_string t =
+      file_string_of_uid t.uid_type
     
+    let to_uid t = t.uid_type
     
     let derive uid =
       match uid.uid_type with
-        Some (Bitprint (sha1, tiger)) ->
+         (Bitprint (sha1, tiger)) ->
           [create (Sha1 (sha1)); create (TigerTree(tiger))]
       | _ -> []
         
@@ -176,7 +190,7 @@ module Uid : sig
     
     let rec expand_rec list uids =
       match list with
-        [] -> []
+        [] -> uids
       | t :: tail ->
           let list = derive t in
           expand_rec (list@tail) (add t uids)
@@ -192,18 +206,7 @@ let string_of_uids list =
   match list with
     [] -> "<unknown>"
   | uid :: _ -> Uid.to_string uid
-  
-type field_name =
-  Field_Size
-| Field_Filename
-| Field_Artist
-| Field_Album
-| Field_Title
-| Field_Format
-| Field_Type
-| Field_Uid  
-| Field_unknown of string
-  
+    
 type file_state =
   FileDownloading
 | FileQueued
@@ -226,41 +229,6 @@ let string_of_state file_state =
 | FileNew -> "FileNew"
 | FileAborted _ -> "FileAborted"
 
-let string_of_field field =
-  match field with
-    Field_Size -> "size"
-  | Field_Filename -> "filename"
-  | Field_Artist -> "artist"
-  | Field_Album -> "album"
-  | Field_Title -> "title"
-  | Field_Format -> "format"
-  | Field_Type -> "type"
-  | Field_Uid -> "uid"
-  | Field_unknown s -> s
-
-let field_of_string s =
-  match String.lowercase s with
-    "size" -> Field_Size
-  | "filename" -> Field_Filename
-  | "artist" -> Field_Artist
-  | "album" -> Field_Album
-  | "title" -> Field_Title
-  | "format" -> Field_Format
-  | "type" -> Field_Type
-  | "uid" -> Field_Uid
-  | _ -> Field_unknown s
-      
-type query =
-  QAnd of query * query
-| QOr of query * query
-| QAndNot of query * query
-| QHasWord of string
-| QHasField of field_name * string
-| QHasMinVal of field_name * int64
-| QHasMaxVal of field_name * int64
-| QNone (** temporary, used when no value is available ;
-	   must be simplified before transforming into strings *)
-
   
   
   
@@ -273,13 +241,45 @@ type tag_value =
 | Addr of Ip.t
 | Uint16 of int
 | Uint8 of int
-
+| Pair of int64 * int64
+  
+type field =
+| Field_Artist (* "Artist" *)
+| Field_Title  (* "Title" *)
+| Field_Album  (* "Album" *)
+| Field_Format (* "format" *)
+| Field_Type   (* "type" *)
+| Field_Length (* "length" *)
+| Field_Bitrate (* "bitrate" *)
+| Field_Codec   (* "codec" *)
+| Field_Availability (* "availability" or "avail" *)
+| Field_Completesources (* "completesources" *)
+| Field_Filename (* "filename" *)
+| Field_Size
+| Field_Uid
+| Field_Filerating
+| Field_Lastseencomplete
+| Field_Mediacodec
+| Field_Medialength
+| Field_UNKNOWN of string
+  
 type tag = {
-    mutable tag_name : string;
+    mutable tag_name : field;
     mutable tag_value : tag_value;
   }
 
   
+type query =
+  QAnd of query * query
+| QOr of query * query
+| QAndNot of query * query
+| QHasWord of string
+| QHasField of field * string
+| QHasMinVal of field * int64
+| QHasMaxVal of field * int64
+| QNone (** temporary, used when no value is available ;
+	   must be simplified before transforming into strings *)
+
   
 type client_type = int
 let client_friend_tag = 1
@@ -336,6 +336,8 @@ type tcp_connection =
 | Connection of TcpBufferedSocket.t (* We are connected *)
 
 type sharing_strategy = {
+    sharing_incoming : bool;
+    sharing_directories : bool;
     sharing_recursive : bool;
     sharing_minsize : int64;
     sharing_maxsize : int64;
@@ -347,8 +349,6 @@ type shared_directory = {
     shdir_priority : int;
     shdir_strategy : string;
     shdir_networks : string list;
-    shdir_upload_lifetime : int;
-    shdir_chunks_percent : int;
   }
   
 type connection_control = {
@@ -358,7 +358,7 @@ type connection_control = {
     mutable control_min_reask : int;
   }
 
-type output_type = TEXT | HTML | ANSI
+type output_type = TEXT | HTML | ANSI | XML | XHTML
   
 type sortvd_type = 
   BySize
@@ -431,6 +431,7 @@ type network_flag =
 | NetworkHasChat
 | NetworkHasSupernodes
 | NetworkHasUpload
+| NetworkHasStats
   
 type network_info = {
     network_netname : string;
@@ -451,13 +452,13 @@ type network = {
     network_name : string;
     network_num : int;
     network_connection_manager : TcpBufferedSocket.connection_manager;
+    network_shortname: string;
     mutable network_flags : network_flag list;
     mutable network_config_file : Options.options_file list;
-    mutable network_incoming_subdir: (unit -> string);
-    mutable network_prefix: (unit -> string);
     mutable op_network_connected_servers : (unit -> server list);
     mutable op_network_is_enabled : (unit -> bool);
     mutable op_network_save_complex_options : (unit -> unit);
+    mutable op_network_save_sources : (unit -> unit);
     mutable op_network_load_complex_options : (unit -> unit);
     mutable op_network_enable : (unit -> unit);
     mutable op_network_disable : (unit -> unit);
@@ -472,7 +473,7 @@ type network = {
       bool -> ((string * Options.option_value) list -> client);
     mutable op_network_recover_temp : (unit -> unit);
     mutable op_network_share : (
-      shared_directory -> string -> string -> int64 -> unit);
+      string -> string -> int64 -> unit);
     mutable op_network_private_message : (string -> string -> unit);
     mutable op_network_parse_url : (string -> bool);
     mutable op_network_connect_servers : (unit -> unit);
@@ -489,6 +490,7 @@ type network = {
     mutable op_network_gui_message : (string -> unit);
     
     mutable op_network_download : (result_info -> file);
+    mutable op_network_display_stats : (Buffer.t -> ui_conn -> unit);
   }
 
 and   ui_user = {
@@ -541,10 +543,147 @@ type avi_info = {
     mutable avi_rate : int;
   }
 
-  
+type ogg_stream_type =
+  OGG_INDEX_STREAM                             (* as per OggDS new Header Format      *)
+| OGG_TEXT_STREAM                              (* as per OggDS new Header Format      *)
+| OGG_AUDIO_STREAM                             (* as per OggDS new Header Format      *)
+| OGG_VORBIS_STREAM                            (* as per Ogg Vorbis I specification   *)
+| OGG_THEORA_STREAM                            (* as per Theora I specification       *)
+| OGG_VIDEO_STREAM                             (* as per OggDS new Header Format      *)
+
+type vorbis_bitrates =
+  Maximum_br of float                          (* maximum bitrate in kbit/s           *)
+| Nominal_br of float                          (* nominal bitrate in kbit/s           *)
+| Minimum_br of float                          (* minimum bitrate in kbit/s           *)
+
+type theora_color_space =
+  CSUndefined
+| CSRec470M
+| CSRec470BG
+
+type ogg_tag =
+  Ogg_codec of string
+| Ogg_bits_per_samples of int                 (* in bits                             *)
+| Ogg_duration of int                         (* duration in seconds                 *)
+| Ogg_has_subtitle
+| Ogg_has_index
+| Ogg_audio_channels of int                   (* number of audio channels            *)
+| Ogg_audio_sample_rate of float              (* in sample/s                         *)
+| Ogg_audio_blockalign of int
+| Ogg_audio_avgbytespersec of float           (* in bytes/s                          *)
+| Ogg_vorbis_version of float                 (* version of vorbis codec. shall be 0 *)
+| Ogg_vorbis_sample_rate of float             (* samplerate in Hertz                 *)
+| Ogg_vorbis_bitrates of vorbis_bitrates list
+| Ogg_vorbis_blocksize_0 of int
+| Ogg_vorbis_blocksize_1 of int
+| Ogg_video_width of float                    (* in pixel                            *)
+| Ogg_video_height of float                   (* in pixel                            *)
+| Ogg_video_sample_rate of float              (* in frames/s                         *)
+| Ogg_aspect_ratio of float
+| Ogg_theora_cs of theora_color_space
+| Ogg_theora_quality of int
+| Ogg_theora_avgbytespersec of int            (* in bytes/s for VBR streams          *)
+
+type ogg_stream_info = {
+  stream_no   : int;
+  stream_type : ogg_stream_type;
+  stream_tags : ogg_tag list;
+}
+
+
+(**********************************************************************************)
+(*                                                                                *)
+(*                  stream_type_to_string                                         *)
+(*                                                                                *)
+(**********************************************************************************)
+
+let stream_type_to_string st =
+  match st with
+    OGG_VIDEO_STREAM -> "video"
+  | OGG_AUDIO_STREAM -> "audio"
+  | OGG_INDEX_STREAM -> "index"
+  | OGG_TEXT_STREAM -> "text"
+  | OGG_VORBIS_STREAM -> "audio"
+  | OGG_THEORA_STREAM -> "video"
+
+(**********************************************************************************)
+(*                                                                                *)
+(*                  vorbis_bitrates_to_string                                     *)
+(*                                                                                *)
+(**********************************************************************************)
+
+let vorbis_bitrates_to_string vb =
+  let s = ref "" in
+  List.iter (fun b ->
+    match b with
+        Maximum_br r -> s := !s ^ (Printf.sprintf "%.0f kbit/s [max]" (r /. 1000.))
+      | Nominal_br r -> s := !s ^ (Printf.sprintf "%.0f kbit/s [nom]" (r /. 1000.))
+      | Minimum_br r -> s := !s ^ (Printf.sprintf "%.0f kbit/s [min]" (r /. 1000.))
+  ) vb;
+  !s
+
+(**********************************************************************************)
+(*                                                                                *)
+(*                  theora_cs_to_string                                           *)
+(*                                                                                *)
+(**********************************************************************************)
+
+let theora_cs_to_string cs =
+  match cs with
+      CSUndefined -> "Undefined"
+    | CSRec470M -> "470M"
+    | CSRec470BG -> "470BG"
+
+(**********************************************************************************)
+(*                                                                                *)
+(*                  ogg_tag_to_string                                             *)
+(*                                                                                *)
+(**********************************************************************************)
+
+let ogg_tag_to_string ogg_tag =
+  match ogg_tag with
+  Ogg_codec s  -> Printf.sprintf "%s [codec]" s
+| Ogg_bits_per_samples n -> Printf.sprintf "%d bits/sample" n
+| Ogg_duration n -> Printf.sprintf "%d s" n
+| Ogg_has_subtitle -> Printf.sprintf "Stream has subtitles"
+| Ogg_has_index -> Printf.sprintf "Stream has chapters"
+| Ogg_audio_channels n -> Printf.sprintf "%d [channels]" n
+| Ogg_audio_sample_rate r -> Printf.sprintf "%.3f samples/s" r
+| Ogg_audio_blockalign n -> Printf.sprintf "block align: %d" n
+| Ogg_audio_avgbytespersec r -> Printf.sprintf "%.0f kbps" (r *. 8. /. 1000.)
+| Ogg_vorbis_version r -> Printf.sprintf "vorbis version: %.0f" r
+| Ogg_vorbis_sample_rate r -> Printf.sprintf "%.0f Hz" r
+| Ogg_vorbis_bitrates l -> Printf.sprintf "%s" (vorbis_bitrates_to_string l)
+| Ogg_vorbis_blocksize_0 n -> Printf.sprintf "blocksize_0: %d" (int_of_float (2. ** float_of_int n))
+| Ogg_vorbis_blocksize_1 n -> Printf.sprintf "blocksize_1: %d" (int_of_float (2. ** float_of_int n))
+| Ogg_video_width r -> Printf.sprintf "width: %.0f pixels" r
+| Ogg_video_height r -> Printf.sprintf "height: %.0f pixels" r
+| Ogg_video_sample_rate r -> Printf.sprintf "%.3f fps" r
+| Ogg_aspect_ratio r -> Printf.sprintf "1:%0.5f" r
+| Ogg_theora_cs cs -> Printf.sprintf "%s" (theora_cs_to_string cs)
+| Ogg_theora_quality n -> Printf.sprintf "%d [quality]" n
+| Ogg_theora_avgbytespersec n -> Printf.sprintf "%d kbps" (n / 1000)
+
+(**********************************************************************************)
+(*                                                                                *)
+(*                  print_ogg_info                                                *)
+(*                                                                                *)
+(**********************************************************************************)
+
+let print_ogg_infos ogg_infos =
+  List.iter (fun stream ->
+    Printf.printf "-== %s info - stream number: %d ==-\n"
+      (stream_type_to_string stream.stream_type) stream.stream_no;
+    List.iter (fun tag ->
+      Printf.printf "  %s\n" (ogg_tag_to_string tag)
+    ) stream.stream_tags
+  ) !ogg_infos;
+  flush stdout
+
 type format =
   AVI of avi_info
 | MP3 of Mp3tag.Id3v1.tag * Mp3tag.info
+| OGG of ogg_stream_info list
 | FormatType of string * string
 | FormatUnknown
 | FormatNotComputed of int
@@ -593,6 +732,7 @@ sending it twice. *)
     mutable gui_rooms : numevents;
     mutable gui_results : numevents;
     mutable gui_shared_files : numevents;
+    mutable gui_networks : numevents;
   
   }
   
@@ -610,6 +750,7 @@ and event =
 | Room_info_event of room
 | Result_info_event of result
 | Shared_info_event of shared
+| Network_info_event of network
 
 | Client_new_file_event of client * string * result
 | File_add_source_event of file * client
@@ -683,12 +824,13 @@ let short_string_of_connection_state s =
     
 exception IgnoreNetwork
   
-let string_of_tag tag =
+let string_of_tag_value tag =
   match tag with
     Uint64 i| Fint64 i -> Int64.to_string i
   | Addr x -> Ip.to_string x
-  | String s -> s
+  | String s -> String.escaped s
   | Uint16 n | Uint8 n -> string_of_int n
+  | Pair (n1, n2) -> Printf.sprintf "%Ld x %Ld" n1 n2
       
 type arg_handler =  ui_conn -> string
 type arg_kind = 
