@@ -43,10 +43,10 @@ let define_expert_option a b ?desc c d e =
 module M = Gui_messages
 module C = Gui_columns
 
-let _ = Unix2.safe_mkdir CommonOptions.config_dir
+let _ = Unix2.safe_mkdir CommonOptions.home_dir
   
 let mldonkey_gui_ini = create_options_file 
-    (Filename.concat CommonOptions.config_dir "mldonkey_newgui.ini")
+    (Filename.concat CommonOptions.home_dir "mldonkey_newgui.ini")
   
 module KeyOption = struct
     
@@ -234,22 +234,22 @@ let color_network_disabled =  define_option mldonkey_gui_section ["colors"; "net
 
 let font_list =  define_option mldonkey_gui_section ["fonts"; "list"]
     ( M.h_font_list) string_option
-     ( if Autoconf.windows then "-unknown-Tahoma-normal-r-normal-*-*-100-*-*-p-*-iso8859-1"
+     ( if Autoconf.system = "windows" then "-unknown-Tahoma-normal-r-normal-*-*-100-*-*-p-*-iso8859-1"
      else "-*-*-*-*-*-*-*-*-*-*-*-*-*-*" )
 
 let font_main_tab =  define_option mldonkey_gui_section ["fonts"; "main_tab"]
     ( M.h_font_main_tab) string_option
-     ( if Autoconf.windows then "-unknown-Tahoma-normal-r-normal-*-*-100-*-*-p-*-iso8859-1"
+     ( if Autoconf.system = "windows" then "-unknown-Tahoma-normal-r-normal-*-*-100-*-*-p-*-iso8859-1"
      else "-*-*-*-*-*-*-*-*-*-*-*-*-*-*" )
 
 let font_networks =  define_option mldonkey_gui_section ["fonts"; "networks"]
     ( M.h_font_networks) string_option
-     ( if Autoconf.windows then "-unknown-Tahoma-normal-r-normal-*-*-100-*-*-p-*-iso8859-1"
+     ( if Autoconf.system = "windows" then "-unknown-Tahoma-normal-r-normal-*-*-100-*-*-p-*-iso8859-1"
      else "-*-*-*-*-*-*-*-*-*-*-*-*-*-*" )
 
 let font_graphic =  define_option mldonkey_gui_section ["fonts"; "graphic"]
     ( M.h_font_graphic)  string_option
-     ( if Autoconf.windows then "-unknown-Tahoma-normal-r-normal-*-*-80-*-*-p-*-iso8859-1"
+     ( if Autoconf.system = "windows" then "-unknown-Tahoma-normal-r-normal-*-*-80-*-*-p-*-iso8859-1"
      else "-*-*-*-*-*-*-*-*-*-*-*-*-*-*" )
 
 (*
