@@ -22,10 +22,7 @@ open CommonOptions
 open Options
 open GnutellaNetwork
   
-let cmd_basedir = Autoconf.current_dir (* will not work on Windows *)
-
-let gnutella_ini = create_options_file (
-    Filename.concat file_basedir GnutellaNetwork.config_file)
+let gnutella_ini = create_options_file GnutellaNetwork.config_file
 
 let gnutella_section = file_section gnutella_ini ["Gnutella"] "Gnutella options"  
 
@@ -43,12 +40,25 @@ let client_port = define_option gnutella_section ["client_port"]
     "The port to bind the client to"
     int_option GnutellaNetwork.port
   
+let gnutella_experimental = define_option gnutella_section
+    ["experimental"]
+    "Activate this option if you want to test experimental features."
+    bool_option false
+
 let gnutella_hostfiles = define_option gnutella_section 
     ["hostfiles"]
   "A list of GWCache urls"
     (list_option string_option)
   [
-    "http://gwebcache.bearshare.net/gcache.php"
+    "http://cache.kicks-ass.net:8000";
+    "http://gwc.jooz.net:8010/gwc";
+    "http://gwc.mine.nu:3333";
+    "http://gwc1.nouiz.org/servlet/GWebCache/req";
+    "http://gwcrab.sarcastro.com:8001";
+    "http://kisama.ath.cx:8080";
+    "http://krill.shacknet.nu:20095/gwc";
+    "http://overbeer.ghostwhitecrab.de";
+    "http://pokerface.bishopston.net:3558";
   ]
 
 let urlfiles = define_option gnutella_section 
@@ -56,7 +66,15 @@ let urlfiles = define_option gnutella_section
   "A list of GWCache urls"
     (list_option string_option)
   [
-    "http://gwebcache.bearshare.net/gcache.php"
+    "http://cache.kicks-ass.net:8000";
+    "http://gwc.jooz.net:8010/gwc";
+    "http://gwc.mine.nu:3333";
+    "http://gwc1.nouiz.org/servlet/GWebCache/req";
+    "http://gwcrab.sarcastro.com:8001";
+    "http://kisama.ath.cx:8080";
+    "http://krill.shacknet.nu:20095/gwc";
+    "http://overbeer.ghostwhitecrab.de";
+    "http://pokerface.bishopston.net:3558";
   ]
   
 let redirectors = define_option gnutella_section 
@@ -67,23 +85,19 @@ let redirectors = define_option gnutella_section
 (* (Ip.of_string "64.61.25.171")   *)
   
   
-let commit_in_subdir = define_option gnutella_section ["commit_in_subdir"]
-  "The subdirectory of temp/ where files should be moved to"
-    string_option "Gnutella"
-
 let user_agent = Printf.sprintf "MLdonkey/%s" Autoconf.current_version
 
 let max_known_ultrapeers = define_option gnutella_section ["max_known_ultrapeers"]
   "Maximal number of ultrapeers remembered"
-    int_option 100
+    int_option 500
 
 let max_known_peers = define_option gnutella_section ["max_known_peers"]
   "Maximal number of peers remembered"
-  int_option 100
+  int_option GnutellaNetwork.max_known_peers_default
     
 let server_connection_timeout = 
   define_option gnutella_section ["server_connection_timeout"] 
-  "timeout when connecting to a server" float_option 10.
+  "timeout when connecting to a server" float_option 30.
 
 let client_uid = define_option gnutella_section ["client_uid"]
     "The UID of this client" Md4.option (Md4.random ())
@@ -105,24 +119,45 @@ let verbose_servers =
   define_option gnutella_section ["verbose_servers"] 
     "level of verbosity when communicating with servers" int_option 0
     *)
-
-let network_options_prefix = define_option gnutella_section
-    ["options_prefix"] "The prefix which is appended to options names
-    when they are used in the telnet/WEB interfaces"
-    string_option GnutellaNetwork.options_prefix
   
 let max_available_slots = define_option gnutella_section
     ["max_available_slots"] "The maximal number of slots for upload by Gnutella clients"
     int_option 5
   
 let shortname o =
-  Printf.sprintf "%s%s" !!network_options_prefix (shortname o)
+  Printf.sprintf "%s%s" GnutellaNetwork.options_prefix (shortname o)
+
+let sha1_verification_threshold = define_option gnutella_section
+    ["sha1_verification_threshold"]
+  "Under this threshold, files are verified using the SHA1 digest
+(always available), without using the tiger-tree (experimental)."
+    int64_option Int64ops.zero
   
 let deflate_connections = define_option gnutella_section 
   ["deflate_connections"]
   "(only for development tests)"  
     bool_option false
   
+let keep_alive = define_option gnutella_section 
+    ["keep_alive"]
+  "(only for development tests)"  
+    bool_option true
+
+let dont_connect = define_option gnutella_section 
+    ["dont_connect"]
+  "(only for development tests)"  
+    bool_option false
+
+let supernode_enabled = define_option gnutella_section 
+  ["supernode_enabled"]
+  "(only for development tests)"  
+    bool_option false
+
+let incoming_data_log = define_option gnutella_section
+    ["incoming_data_log"]
+    "(only for development tests)"
+    string_option ""
+  
 let gui_gnutella_options_panel = 
   (*
   define_option gnutella_section ["gui_gnutella_options_panel"]
@@ -135,7 +170,6 @@ let gui_gnutella_options_panel =
 (*    "Max Connected Ultrapeers", shortname max_ultrapeers, "T"; 
     "Max Known Ultrapeers", shortname max_known_ultrapeers, "T";
     "Max Known Peers", shortname max_known_peers, "T";    *)
-    "Commit Downloads In Incoming Subdir", shortname commit_in_subdir, "T";
     "Max Available Slots", shortname max_available_slots, "T";
     
     "          Max Connected Ultrapeers", shortname max_ultrapeers, "T"; 
