@@ -28,6 +28,7 @@ let chunk_min_size = ref (Int64.of_int 65000)
 
 let max_buffered = ref (Int64.of_int (1024*1024))
 
+let create_dir_mask = ref "755"
 let verbose = false
 let max_cache_size = ref 50
 
@@ -42,10 +43,7 @@ let rw_flag =  [Unix.O_CREAT; Unix.O_RDWR]
 let really_write fd s pos len =
   try
     Unix2.really_write fd s pos len
-  with e ->
-      lprintf "Exception in really_write: pos=%d len=%d, string length=%d\n"
-        pos len (String.length s);
-      raise e
+  with e -> raise e
 
 module FDCache = struct
     
@@ -99,17 +97,21 @@ module FDCache = struct
             close t
 
 
-    let local_force_fd t =
+    let local_force_fd t writable =
       check_destroyed t;
       let fd =
         match t.fd with
           None ->
-            if !cache_size >= !max_cache_size then  close_one ();
+            if !cache_size >= !max_cache_size then close_one ();
             let fd =
-              try
-                Unix.openfile t.filename rw_flag rights
-              with Unix.Unix_error( (Unix.EACCES | Unix.EROFS) ,_,_) ->
+              if writable then begin
+                try
+                  Unix.openfile t.filename rw_flag rights
+                with Unix.Unix_error( (Unix.EACCES | Unix.EROFS) ,_,_) ->
                   Unix.openfile t.filename ro_flag 0o400
+                end
+              else
+                Unix.openfile t.filename ro_flag 0o400
             in
             incr cache_size;
 (*            lprintf "local_force: opening %d\n" (Obj.magic fd);  *)
@@ -132,13 +134,25 @@ module FDCache = struct
       Unix2.rename t.filename f;
       destroy t
 
+    let multi_rename t f file =
+      check_destroyed t;
+      close t;
+      (let d = (Filename.dirname (Filename.concat f file)) in
+        Unix2.safe_mkdir d;
+	Unix.chmod d (Misc.int_of_octal_string !create_dir_mask);
+	Unix2.can_write_to_directory d);
+      Unix2.rename t.filename (Filename.concat f file);
+      destroy t
+
     let ftruncate64 t len =
       check_destroyed t;
-      Unix2.c_ftruncate64 (local_force_fd t) len
+      Unix2.c_ftruncate64 (local_force_fd t true) len
 
-    let getsize64 t = 
+    let getsize64 t writable =
       check_destroyed t;
-      Unix2.c_getsize64 t.filename
+      let s = Unix2.c_getfdsize64 (local_force_fd t writable) in
+      if not writable then close t;
+      s
 
     let mtime64 t =
       check_destroyed t;
@@ -155,16 +169,28 @@ module FDCache = struct
       destroy t
 
     let read file file_pos string string_pos len =
-      let fd = local_force_fd file in
+      let fd = local_force_fd file true in
+      (*
+        I know I am calling local_force_fd with writable = true
+        although we are reading here, but I had problems with
+        it like exceptions in really_write
+      *)
       let final_pos = Unix2.c_seek64 fd file_pos Unix.SEEK_SET in
       if verbose then lprintf "really_read %d\n" len;
       Unix2.really_read fd string string_pos len
 
     let write file file_pos string string_pos len =
-      let fd = local_force_fd file in
+      let fd = local_force_fd file true in
       let final_pos = Unix2.c_seek64 fd file_pos Unix.SEEK_SET in
       if verbose then lprintf "really_write %d\n" len;
-      really_write fd string string_pos len
+      begin
+        try
+	  really_write fd string string_pos len
+	with e ->
+	    lprintf_nl "[Unix32] Exception %s in write: file %s file_pos=%Ld len=%d string_pos=%d, string length=%d"
+	      (Printexc2.to_string e) file.filename file_pos len string_pos (String.length string);
+	    raise e
+      end
 
     let copy_chunk t1 t2 pos1 pos2 len64 =
       check_destroyed t1;
@@ -188,12 +214,12 @@ module FDCache = struct
 module type File =   sig
     type t
     val create : string -> t
-    val fd_of_chunk : t -> int64 -> int64 ->
-      Unix.file_descr * int64 * string option
+    val apply_on_chunk : t -> int64 -> int64 ->
+      (Unix.file_descr -> int64 -> 'a) -> 'a
     val close : t -> unit
     val rename : t -> string -> unit
     val ftruncate64 : t -> int64 -> unit
-    val getsize64 : t -> int64
+    val getsize64 : t -> bool -> int64
     val mtime64 : t -> float
     val exists : t -> bool
     val remove : t -> unit
@@ -210,9 +236,9 @@ module DiskFile = struct
 
     let create = FDCache.create
 
-    let fd_of_chunk t pos_s len_s =
-      let fd = FDCache.local_force_fd t in
-      fd, pos_s, None
+    let apply_on_chunk t pos_s len_s f =
+      let fd = FDCache.local_force_fd t true in
+      f fd pos_s
 
     let close = FDCache.close
 
@@ -300,6 +326,7 @@ module MultiFile = struct
 
     let create dirname files =
       Unix2.safe_mkdir dirname;
+      Unix2.can_write_to_directory dirname;
       let rec iter files pos files2 =
         match files with
           [] ->
@@ -320,14 +347,15 @@ module MultiFile = struct
         | (filename, size) :: tail ->
             let temp_filename = Filename.concat dirname filename in
             Unix2.safe_mkdir (Filename.dirname temp_filename);
+	    Unix2.can_write_to_directory (Filename.dirname temp_filename);
             let fd = FDCache.create temp_filename in
-            let _ = FDCache.local_force_fd fd in
+            let _ = FDCache.local_force_fd fd true in
             iter tail (pos ++ size)
             ({
                 filename = filename;
                 pos = pos;
                 len = size;
-                current_len = FDCache.getsize64 fd;
+                current_len = FDCache.getsize64 fd true;
                 fd = fd;
                 tail = [];
               } :: files2)
@@ -366,14 +394,14 @@ module MultiFile = struct
         (max_len -- max_possible_write)
 
 
-    let fd_of_chunk t chunk_begin chunk_len =
+    let apply_on_chunk t chunk_begin chunk_len f =
       let (file, tail) = find_file t chunk_begin in
       let chunk_end = chunk_begin ++ chunk_len in
       let file_begin = file.pos in
       let max_current_pos = file_begin ++ file.current_len in
       if max_current_pos >= chunk_end then
-        let fd = FDCache.local_force_fd file.fd in
-        fd, chunk_begin -- file_begin, None
+        let fd = FDCache.local_force_fd file.fd true in
+        f fd (chunk_begin -- file_begin)
       else
       let temp_file = Filename.temp_file "chunk" ".tmp" in
       let file_out = FDCache.create temp_file in
@@ -401,9 +429,14 @@ module MultiFile = struct
           file_pos ++ zeros
       );
       FDCache.close file_out;
-      let fd = FDCache.local_force_fd file_out in
-      fd, zero, Some temp_file
-
+      let fd = FDCache.local_force_fd file_out true in
+      try 
+        let v = f fd zero in
+        Sys.remove temp_file;
+        v
+      with e -> 
+          Sys.remove temp_file;
+          raise e
 
     let close t =
       List.iter (fun file -> FDCache.close file.fd) t.files
@@ -413,13 +446,7 @@ module MultiFile = struct
 
     let rename t f =
       close t;
-      Unix2.rename t.dirname f
-
-      (*
-      List.iter (fun file ->
-          file.fd.FDCache.filename <- Filename.concat t.dirname file.filename
-      ) t.files
-*)
+      List.iter (fun file -> FDCache.multi_rename file.fd f file.filename) t.files
 
     let ftruncate64 t size =
       t.size <- size
@@ -491,7 +518,10 @@ module MultiFile = struct
       let len64 = Int64.of_int len in
       t.size <- max t.size (chunk_begin ++ len64);
       let time = Unix.time () in
-      Unix.utimes t.dirname time time;
+      (* this does not work on win32 because of
+         file locking i guess, therefore ignore
+         all exceptions *)
+      (try Unix.utimes t.dirname time time with _ -> ());
       ()
 
   end
@@ -528,6 +558,7 @@ module SparseFile = struct
       let dirname = filename ^ ".chunks" in
 (*      lprintf "Creating directory %s\n" dirname; *)
       Unix2.safe_mkdir dirname;
+      Unix2.can_write_to_directory dirname;
       {
         filename = filename;
         dirname = dirname;
@@ -626,7 +657,7 @@ module SparseFile = struct
     
     let build t = ()
     
-    let fd_of_chunk t chunk_begin chunk_len =
+    let apply_on_chunk t chunk_begin chunk_len f =
       
       let index = find_read_pos t chunk_begin in
       let nchunks = Array.length t.chunks in
@@ -639,8 +670,8 @@ module SparseFile = struct
         
         let chunk = t.chunks.(index) in
         let in_chunk_pos = chunk_begin -- chunk.pos in
-        let fd = FDCache.local_force_fd chunk.fd in
-        fd, in_chunk_pos, None
+        let fd = FDCache.local_force_fd chunk.fd true in
+        f fd in_chunk_pos
       
       else
       
@@ -679,8 +710,14 @@ module SparseFile = struct
       iter zero (find_read_pos t chunk_begin) chunk_begin chunk_len;
       
       FDCache.close file_out;
-      let fd = FDCache.local_force_fd file_out in
-      fd, zero, Some temp_file
+      let fd = FDCache.local_force_fd file_out true in
+      try
+        let v = f fd zero in
+        Sys.remove temp_file;
+        v
+      with e -> 
+          Sys.remove temp_file;
+          raise e
     
     let close t =
       Array.iter (fun file -> FDCache.close file.fd) t.chunks
@@ -743,7 +780,7 @@ module SparseFile = struct
     let ftruncate64 t size =
       t.size <- size
 
-    let getsize64 t = t.size
+    let getsize64 t writable = t.size
 
     let mtime64 t =
       let st = Unix.LargeFile.stat t.dirname in
@@ -864,7 +901,10 @@ module SparseFile = struct
       t.size <- max t.size (chunk_begin ++ len64);
 
       let time = Unix.time () in
-      Unix.utimes t.dirname time time
+      (* this does not work on win32 because of
+         file locking i guess, therefore ignore
+         all exceptions *)
+      (try Unix.utimes t.dirname time time with _ -> ())
   end
 
 
@@ -940,11 +980,12 @@ let mtime64 t =
   | SparseFile t -> SparseFile.mtime64 t
   | Destroyed -> failwith "Unix32.mtime64 on destroyed FD"
       
-let getsize64 t =
+let getsize64 t writable =
   match t.file_kind with
-  | DiskFile t -> DiskFile.getsize64 t
+  | DiskFile t -> DiskFile.getsize64 t writable
+    (* only avoid opening rw on shared files, shared files can only be DiskFile *)
   | MultiFile t -> MultiFile.getsize64 t
-  | SparseFile t -> SparseFile.getsize64 t
+  | SparseFile t -> SparseFile.getsize64 t writable
   | Destroyed -> failwith "Unix32.getsize64 on destroyed FD"
 
 let fds_size = Unix2.c_getdtablesize ()
@@ -985,7 +1026,7 @@ let buffer = Buffer.create 65000
 let flush_buffer t offset =
   if verbose then lprintf "flush_buffer\n";
   let s = Buffer.contents buffer in
-  Buffer.clear buffer;
+  Buffer.reset buffer;
   let len = String.length s in
   try
     if verbose then lprintf "seek64 %Ld\n" offset;
@@ -1017,7 +1058,7 @@ let flush_fd t =
     match t.buffers with
       [] -> ()
     | (s, pos_s, len_s, offset, len) :: tail ->
-        Buffer.clear buffer;
+        Buffer.reset buffer;
         Buffer.add_substring buffer s pos_s len_s;
         t.buffers <- tail;
         iter_in offset len
@@ -1058,7 +1099,9 @@ let read t file_pos string string_pos len =
   | DiskFile t -> DiskFile.read t file_pos string string_pos len
   | MultiFile t -> MultiFile.read t file_pos string string_pos len
   | SparseFile t -> SparseFile.read t file_pos string string_pos len
-  | Destroyed -> failwith "Unix32.read on destroyed FD"
+  | Destroyed -> failwith 
+        (Printf.sprintf "Unix32.read on destroyed FD %s"
+          (filename t))
       
 let flush _ =
   try
@@ -1152,14 +1195,14 @@ let destroy t =
 let create_rw filename = create_diskfile filename rw_flag 0o666
 let create_ro = create_rw
 
-let fd_of_chunk t pos len =
+let apply_on_chunk t pos len f =
   match t.file_kind with
-  | DiskFile t -> DiskFile.fd_of_chunk t pos len
+  | DiskFile t -> DiskFile.apply_on_chunk t pos len f
   | MultiFile tt ->
       flush_fd t;
-      MultiFile.fd_of_chunk tt pos len
-  | SparseFile t -> SparseFile.fd_of_chunk t pos len
-  | Destroyed -> failwith "Unix32.fd_of_chunk on destroyed FD"
+      MultiFile.apply_on_chunk tt pos len f
+  | SparseFile t -> SparseFile.apply_on_chunk t pos len f
+  | Destroyed -> failwith "Unix32.apply_on_chunk on destroyed FD"
       
 let exists t =
   flush_fd t;
@@ -1178,7 +1221,7 @@ let remove t =
   | SparseFile t -> SparseFile.remove t
   | Destroyed -> failwith "Unix32.remove on destroyed FD"
       
-let getsize s =  getsize64 (create_ro s)
+let getsize s writable =  getsize64 (create_ro s) writable
 let mtime s =  mtime64 (create_ro s)
 
 let file_exists s = exists (create_ro s)
@@ -1296,7 +1339,7 @@ The following functions have to be rewritten:
 
 (* TODO: there is no need to create a temporary file when the wanted chunk
 overlaps different parts, but these parts are on the same physical file. *)
-    let fd_of_chunk t chunk_begin chunk_len =
+    let apply_on_chunk t chunk_begin chunk_len f =
       let chunk_end = chunk_begin ++ chunk_len in
       let rec iter list =
         match list with 
@@ -1304,7 +1347,7 @@ overlaps different parts, but these parts are on the same physical file. *)
         | part :: tail ->
             if part.part_begin <= chunk_begin &&
               part.part_end >= chunk_end then
-              fd_of_chunk part.part_file chunk_begin chunk_len              
+              apply_on_chunk part.part_file chunk_begin chunk_len f
             else
             if part.part_end > chunk_begin then
               make_temp_file list
@@ -1329,8 +1372,13 @@ overlaps different parts, but these parts are on the same physical file. *)
         in
         fill zero chunk_begin chunk_len list;
         old_close file_out;
-        let fd, _, _ = fd_of_chunk file_out zero chunk_len in
-        fd, zero, Some temp_file
+        try
+          let v = apply_on_chunk file_out zero chunk_len f in
+          Sys.remove temp_file;
+          v
+        with e ->
+            Sys.remove temp_file;
+            raise e
       
       in
       
@@ -1375,5 +1423,186 @@ when these two files have already been partially downloaded ? This
       ()
       
   end
+let destroyed t = t.file_kind = Destroyed
   
 type t = file
+
+let bad_fd = 
+  let t = create_rw "/dev/null" in
+  t.file_kind <- Destroyed;
+  t
+
+type statfs = {
+  f_type : int64;   (* type of filesystem *)
+  f_bsize : int64;  (* optimal transfer block size *)
+  f_blocks : int64; (* total data blocks in file system *)
+  f_bfree : int64;  (* free blocks in fs *)
+  f_bavail : int64; (* free blocks avail to non-superuser *)
+  f_files : int64;  (* total file nodes in file system *)
+  f_ffree : int64;  (* free file nodes in fs *)
+  f_fsid : unit;  (* See note in statfs(2) *)
+  f_fnamelen : int64; (* maximum length of filenames *)
+  f_basetype : string; (* type of filesystem - Solaris, (-1) on other systems, use f_type there *)
+  f_frsize : int64;  (* Fundamental file system block size, (-1) if not provided by system *)
+}
+
+exception Not_supported
+exception Error
+
+let _ = Callback.register_exception "not supported" Not_supported
+let _ = Callback.register_exception "error" Error
+
+external statfs : string -> statfs = "statfs_statfs"
+
+let ( ** ) x y = Int64.mul x y
+let ( ++ ) x y = Int64.add x y
+let ( -- ) x y = Int64.sub x y
+let ( // ) x y = Int64.div x y
+
+let bsize dir =
+  begin
+    try
+  let s = statfs dir in
+    if s.f_frsize = Int64.zero || s.f_frsize = Int64.of_int (-1) then
+      s.f_bsize
+    else
+      s.f_frsize
+    with e -> Int64.of_int (-1)
+  end
+
+let blocks dir =
+  begin
+    try
+  let s = statfs dir in
+    s.f_blocks
+    with e -> Int64.of_int (-1)
+  end
+
+let bfree dir =
+  begin
+    try
+  let s = statfs dir in
+    s.f_bfree
+    with e -> Int64.of_int (-1)
+  end
+
+let bavail dir =
+  begin
+    try
+  let s = statfs dir in
+    s.f_bavail
+    with e -> Int64.of_int (-1)
+  end
+
+let fnamelen dir =
+  begin
+    try
+  let s = statfs dir in
+    s.f_fnamelen
+    with e -> Int64.of_int (-1)
+  end
+
+let disktotal dir =
+(* total disk space in bytes *)
+  begin
+    try
+  let s = statfs dir in
+    ((bsize dir) ** s.f_blocks)
+    with e -> Int64.of_int (-1)
+  end
+
+let diskfree dir =
+(* free disk space in bytes *)
+  begin
+    try
+  let s = statfs dir in
+    ((bsize dir) ** s.f_bavail)
+    with e -> Int64.of_int (-1)
+  end
+
+let diskused dir =
+(* used disk space in bytes *)
+  begin
+    try
+  let s = statfs dir in
+    ((bsize dir) ** (s.f_blocks -- s.f_bavail))
+    with e -> Int64.of_int (-1)
+  end
+
+let percentused dir =
+(* percentage of used disk space *)
+  if (diskfree dir) = Int64.of_int (-1) then
+    (-1)
+  else
+    Int64.to_int (100L -- ((diskfree dir) ** 100L // (disktotal dir)))
+
+let percentfree dir =
+(* percentage of free disk space *)
+  if (diskfree dir) = Int64.of_int (-1) then
+    (-1)
+  else
+    Int64.to_int ((diskfree dir) ** 100L // (disktotal dir))
+
+let filesystem dir =
+  begin
+    try
+  let s = statfs dir in
+    match s.f_type with
+(* values copied from statfs(2) manpage *)
+    | 0xadf5L -> "ADFS_SUPER_MAGIC"
+    | 0xADFFL -> "AFFS_SUPER_MAGIC"
+    | 0x42465331L -> "BEFS_SUPER_MAGIC"
+    | 0x1BADFACEL -> "BFS_MAGIC"
+    | 0xFF534D42L -> "CIFS_MAGIC_NUMBER"
+    | 0x73757245L -> "CODA_SUPER_MAGIC"
+    | 0x012FF7B7L -> "COH_SUPER_MAGIC"
+    | 0x28cd3d45L -> "CRAMFS_MAGIC"
+    | 0x1373L -> "DEVFS_SUPER_MAGIC"
+    | 0x00414A53L -> "EFS_SUPER_MAGIC"
+    | 0x137DL -> "EXT_SUPER_MAGIC"
+    | 0xEF51L -> "ext2" (* EXT2_OLD_SUPER_MAGIC *)
+    | 0xEF53L -> "ext2/3" (* EXT2/3_SUPER_MAGIC *)
+    | 0x4244L -> "HFS_SUPER_MAGIC"
+    | 0xF995E849L -> "HPFS_SUPER_MAGIC"
+    | 0x958458f6L -> "HUGETLBFS_MAGIC"
+    | 0x9660L -> "ISOFS_SUPER_MAGIC"
+    | 0x4000L -> "ISOFS_SUPER_MAGIC_WIN" (* from coreutils-5.2.1, stat.c *)
+    | 0x4004L -> "ISOFS_SUPER_MAGIC_R_WIN" (* from coreutils-5.2.1, stat.c *)
+    | 0x72b6L -> "JFFS2_SUPER_MAGIC"
+    | 0x3153464aL -> "JFS_SUPER_MAGIC"
+    | 0x137FL -> "MINIX_SUPER_MAGIC"
+    | 0x138FL -> "MINIX_SUPER_MAGIC2"
+    | 0x2468L -> "MINIX2_SUPER_MAGIC"
+    | 0x2478L -> "MINIX2_SUPER_MAGIC2"
+    | 0x4d44L -> "msdos" (* MSDOS_SUPER_MAGIC *)
+    | 0x4006L -> "fat" (* from coreutils-5.2.1, stat.c *)
+    | 0x564cL -> "NCP_SUPER_MAGIC"
+    | 0x6969L -> "NFS_SUPER_MAGIC"
+    | 0x5346544eL -> "ntfs" (* NTFS_SB_MAGIC *)
+    | 0x9fa1L -> "OPENPROM_SUPER_MAGIC"
+    | 0x9fa0L -> "PROC_SUPER_MAGIC"
+    | 0x002fL -> "QNX4_SUPER_MAGIC"
+    | 0x52654973L -> "reiserfs" (* REISERFS_SUPER_MAGIC *)
+    | 0x52345362L -> "reiser4"
+    | 0x7275L -> "ROMFS_MAGIC"
+    | 0x517BL -> "smb" (* SMB_SUPER_MAGIC *)
+    | 0x012FF7B6L -> "SYSV2_SUPER_MAGIC"
+    | 0x012FF7B5L -> "SYSV4_SUPER_MAGIC"
+    | 0x01021994L -> "tmpfs" (* TMPFS_MAGIC *)
+    | 0x15013346L -> "UDF_SUPER_MAGIC"
+    | 0x00011954L -> "UFS_MAGIC"
+    | 0x9fa2L -> "USBDEVICE_SUPER_MAGIC"
+    | 0xa501FCF5L -> "VXFS_SUPER_MAGIC"
+    | 0x012FF7B4L -> "XENIX_SUPER_MAGIC"
+    | 0x58465342L -> "XFS_SUPER_MAGIC"
+    | 0x012FD16DL -> "_XIAFS_SUPER_MAGIC"
+    | 5L -> "iso9660" (* Cygwin *)
+    | 6L -> "fat" (* Cygwin *)
+    | 0x700FFL -> "ntfs" (* Cygwin *)
+    | 0xC3L -> "ext2/3" (* Cygwin *)
+    | _ -> if s.f_basetype <> "-1" then
+	     s.f_basetype
+	   else
+	     Printf.sprintf "unknown (%Ld)" s.f_type
+    with e -> "not supported"
+  end
